@@ -1,8 +1,7 @@
 #!/usr/bin/env python
 """A top-like program for monitoring ssh tunnels or any tunnels"""
-# TODO- the disabled coloring is not working
 # TODO- quit doesnt work
-# TODO- we are reviving dead tunnels
+# TODO- we are not reviving dead tunnels
 import argparse
 import asyncio
 import copy
@@ -157,29 +156,27 @@ def render(
     iterator = iter(lines)
     stdscr.addstr(1, 1, lines[0], curses.color_pair(3))
     next(iterator)
-    for i, line in enumerate(iterator):
-        try:
-            line_content = stdscr.instr(sel + 2, 1).decode("utf-8")
-            name: str = line_content[: line_content.find(" ")]
-        finally:
-            name = ""
+    name_list = {}
+    for task in tasks:
+        name_list[task.get_name()] = True
+    for i, (line, name) in enumerate(zip(iterator, data_cols.keys())):
         if i == sel:
             stdscr.addstr(
                 (2 + i) % (len(lines) + 1),
                 1,
                 line,
-                curses.color_pair(2)
-                if name not in tasks
-                else curses.color_pair(5),
+                curses.color_pair(5)
+                if name not in name_list
+                else curses.color_pair(2),
             )
         else:
             stdscr.addstr(
                 2 + i,
                 1,
                 line,
-                curses.color_pair(1)
-                if name not in tasks
-                else curses.color_pair(4),
+                curses.color_pair(4)
+                if name not in name_list
+                else curses.color_pair(1),
             )
         stdscr.addstr("\n")
     stdscr.box()
@@ -213,7 +210,7 @@ class TunnelManager:
             str, typing.Dict[str, str]
         ] = self.read_conf()
         self.tunnel_tasks: typing.List[asyncio.Task] = []
-        self.tunnel_test_tasks: typing.List[asyncio.Task] = []
+        self.tunnel_test_tasks: typing.Dict[str, asyncio.Task] = {}
         self.scheduler_task: asyncio.Task
         self.scheduler_table: typing.Dict[
             str, int
@@ -240,6 +237,9 @@ class TunnelManager:
         """Remove the reference"""
         delete_index: int = -1
         delete_task.cancel()
+        self.data_cols[delete_task.get_name()]["status"] = "UNKWN"
+        self.data_cols[delete_task.get_name()]["stdout"] = "n/a"
+        self.data_cols[delete_task.get_name()]["stderr"] = "n/a"
         self.write_log(f"{delete_task.get_name()} is being cancelled\n")
         await asyncio.sleep(0)
         for i, task in enumerate(task_list):
@@ -263,13 +263,13 @@ class TunnelManager:
                     "address": value["address"],
                     "port": value["port"],
                     "command": value["command"],
-                    "status": "UNKN",
+                    "status": "UNKWN",
                     "test_command": value["test_command"],
                     "test_command_result": value["test_command_result"],
                     "test_interval": value["test_interval"],
                     "test_timeout": value["test_timeout"],
-                    "stdout": "",
-                    "stderr": "",
+                    "stdout": "n/a",
+                    "stderr": "n/a",
                 }
         return data_cols
 
@@ -290,12 +290,15 @@ class TunnelManager:
             proc.terminate()
             raise
 
-    async def run_test_coro(
-        self, cmd: str, task_name: str
-    ) -> typing.Tuple[bytes, bytes]:
+    async def run_test_coro(self, cmd: str, task_name: str) -> None:
         """Run a test command"""
         try:
-            stdout, stderr = await self.run_subprocess(cmd)
+            async with asyncio.timeout_at(
+                asyncio.get_event_loop().time()
+                + float(self.data_cols[task_name]["test_timeout"])
+            ):
+                self.write_log("running test for " + task_name + "\n")
+                stdout, stderr = await self.run_subprocess(cmd)
             stdout_str: str = stdout.decode("utf-8").strip("\n").strip('"')
             stderr_str: str = stderr.decode("utf-8").strip("\n").strip('"')
 
@@ -306,11 +309,15 @@ class TunnelManager:
             else:
                 self.data_cols[task_name]["status"] = "DOWN"
 
-            return stdout, stderr
+            del self.tunnel_test_tasks[task_name]
+            # return stdout, stderr
         except asyncio.TimeoutError:
+            self.write_log(f"test for {task_name} timed out\n")
             self.data_cols[task_name]["status"] = "TMOUT"
             self.data_cols[task_name]["stdout"] = "-"
             self.data_cols[task_name]["stderr"] = "-"
+            self.tunnel_test_tasks[task_name].cancel()
+            del self.tunnel_test_tasks[task_name]
             raise
 
     async def tunnel_procs(
@@ -353,8 +360,6 @@ class TunnelManager:
                         if task.get_name() == k:
                             delete_task = task
                             break
-                            # task.cancel()
-                            # await asyncio.sleep(0)
 
                     if delete_task is not None:
                         await self.stop_task(delete_task, self.tunnel_tasks)
@@ -373,8 +378,6 @@ class TunnelManager:
             if k not in data_cols_new:
                 for task in self.tunnel_tasks:
                     if task.get_name() == k:
-                        # task.cancel()
-                        # await asyncio.sleep(0)
                         delete_task = task
                         break
                 if delete_task is not None:
@@ -394,7 +397,7 @@ class TunnelManager:
     def write_log(self, log: str):
         """A simple logger"""
         with open(
-            "/home/devi/devi/abbatoir/hole15/log",
+            os.path.expanduser("~/.tunneltoplog"),
             "a",
             encoding="utf-8",
         ) as logfile:
@@ -403,49 +406,38 @@ class TunnelManager:
     async def restart_task(self, line_content: str) -> None:
         """restart a task"""
         name: str = line_content[: line_content.find(" ")]
-        # was_cancelled: bool = False
         for task in self.tunnel_tasks:
             if task.get_name() == name:
-                # was_cancelled = task.cancel()
-                # self.write_log(f"was_cancelled: {was_cancelled}")
                 await self.stop_task(task, self.tunnel_tasks)
-                # await task
-                # await asyncio.sleep(0)
-                for _, value in self.data_cols.items():
-                    if value["name"] == name and task.cancelled():
-                        self.tunnel_tasks.append(
-                            asyncio.create_task(
-                                self.run_subprocess(value["command"]),
-                                name=value["name"],
-                            )
-                        )
-                        await asyncio.sleep(0)
+                tunnel_entry = self.data_cols[name]
+                self.tunnel_tasks.append(
+                    asyncio.create_task(
+                        self.run_subprocess(tunnel_entry["command"]),
+                        name=tunnel_entry["name"],
+                    )
+                )
+                await asyncio.sleep(0)
+                break
 
     async def flip_task(self, line_content: str) -> None:
         """flip a task"""
         name: str = line_content[: line_content.find(" ")]
-        # was_cancelled: bool = False
         was_active: bool = False
         for task in self.tunnel_tasks:
             if task.get_name() == name:
                 await self.stop_task(task, self.tunnel_tasks)
-                # was_cancelled = task.cancel()
-                # await asyncio.sleep(0)
-                # self.write_log(f"was_cancelled: {was_cancelled}")
-                # await task
                 was_active = True
                 break
 
         if not was_active:
-            for _, value in self.data_cols.items():
-                if value["name"] == name:
-                    self.tunnel_tasks.append(
-                        asyncio.create_task(
-                            self.run_subprocess(value["command"]),
-                            name=value["name"],
-                        )
-                    )
-                    await asyncio.sleep(0)
+            tunnel_entry = self.data_cols[name]
+            self.tunnel_tasks.append(
+                asyncio.create_task(
+                    self.run_subprocess(tunnel_entry["command"]),
+                    name=tunnel_entry["name"],
+                )
+            )
+            await asyncio.sleep(0)
 
     async def quit(self) -> None:
         """Cleanly quit the applicaiton"""
@@ -469,30 +461,34 @@ class TunnelManager:
                     return
                 for key, value in self.scheduler_table.items():
                     if value == 0 and key not in self.tunnel_test_tasks:
+                        self.write_log("rescheduling test for " + key + "\n")
                         tunnel_entry = self.data_cols[key]
                         test_task = asyncio.create_task(
-                            asyncio.wait_for(
-                                self.run_test_coro(
-                                    tunnel_entry["test_command"],
-                                    tunnel_entry["name"],
-                                ),
-                                timeout=float(tunnel_entry["test_timeout"]),
+                            self.run_test_coro(
+                                tunnel_entry["test_command"],
+                                tunnel_entry["name"],
                             ),
                             name=key,
                         )
-                        self.tunnel_test_tasks.append(test_task)
-                        self.scheduler_table[key] = int(
-                            tunnel_entry["test_interval"]
-                        )
                         await asyncio.sleep(0)
-                    else:
+                        self.tunnel_test_tasks[key] = test_task
+                    if value > 0:
                         self.scheduler_table[key] = (
                             self.scheduler_table[key] - 1
                         )
+                    if value <= 0:
+                        self.write_log("revitalizing test for " + key + "\n")
+                        self.scheduler_table[key] = int(
+                            self.data_cols[key]["test_interval"]
+                        )
 
                 # we are using a 1 second ticker. basically the scheduler
                 # runs every second instead of as fast as it can
                 await asyncio.sleep(1)
+                for test_task_name in self.tunnel_test_tasks.keys():
+                    self.write_log(test_task_name + " ")
+                self.write_log(repr(self.scheduler_table))
+                self.write_log("\n")
         except asyncio.CancelledError:
             pass
 
@@ -542,9 +538,6 @@ class TunnelManager:
             stdscr.keypad(False)
             curses.echo()
             curses.endwin()
-            # tasks = asyncio.all_tasks()
-            # for task in tasks:
-            #     task.cancel()
             await self.quit()
 
 
